package ca.gui;

import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

import javax.swing.text.AttributeSet;
import javax.swing.JTextField;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

public class JTextFieldClearOnFocus extends JTextField implements FocusListener {
	
	private static final long serialVersionUID = 1L;

	public JTextFieldClearOnFocus(){
		super();
		addFocusListener(this);
		setDocument(new NumericAndXDocument());
		setText("0");
		setHorizontalAlignment(JTextField.CENTER);
	}
	
	public JTextFieldClearOnFocus(ActionListener al){
		this();
		addActionListener(al);
	}

	@Override
	public void focusGained(FocusEvent arg0) {
		selectAll();
	}

	@Override
	public void focusLost(FocusEvent arg0) {
		
	}
	
	public class NumericAndXDocument extends PlainDocument {
		private static final long serialVersionUID = -9055789073302565261L;

		// Constructor
		public NumericAndXDocument() {
			super();
		}

		// Insert string method
		public void insertString(int offset, String str, AttributeSet attr) throws BadLocationException {
			if (str != null) {
				if (StringFormat.isNumeric(str) == false && str.equals("x") == false) {
					// First, is it a valid character?
					Toolkit.getDefaultToolkit().beep();
					return;
				} else if ( super.getLength() > 0 && super.getText(0, super.getLength()).contains("x") == true) {
					// Can't be x and number
					Toolkit.getDefaultToolkit().beep();
					return;
				}

				// All is fine, so add the character to the text box
				super.insertString(offset, str, attr);
			}
			return;
		}
	}

	public static class StringFormat {
		public static boolean isNumeric(String str) {
			try {
				int x = Integer.parseInt(str);
				x++;
				return true;
			} catch (NumberFormatException nFE) {
				return false;
			}
		}

	}

}
