package ca.gui;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;

import mechanics.Game;

import org.eclipse.wb.swing.FocusTraversalOnArray;

import ca.RuleBook;
import ca.rules.HashRule;

public class RuleEditor extends JFrame implements ActionListener,TreeSelectionListener {
	
	private static final long serialVersionUID = 1L;

	private JButton saveRuleButton;
	private JButton deleteRuleButton;
	
	private JTextFieldClearOnFocus[][][] textFields;
	
	private RuleBook rulebook;
	private Game game;

	private JTextField ruleName;
	private JTextFieldClearOnFocus becomeField;

	private DefaultTreeModel treeModel;

	private HashMap<DefaultMutableTreeNode, HashRule> mapHashRuleNodeLookup;
	private HashMap<String, DefaultMutableTreeNode> mapHashNodeLookup;
	private HashMap<Integer, DefaultMutableTreeNode> availableRootNodes;

	private JTree treeObject;
	
	private String[] ruleFiles = { "rules.txt", "subrules.txt" };

	private JComboBox ruleFileList;

	private DefaultMutableTreeNode treeRootNode;

	public RuleEditor(Visualizer visulizer){
		
		if(visulizer != null){
			game = visulizer.getGame();
			rulebook = game.getRuleBook();
		} else {
			game = new Game(null);
			rulebook = new RuleBook(null);
		}
		
		// Make it possible to find HashRule based on Node
		mapHashRuleNodeLookup = new HashMap<DefaultMutableTreeNode, HashRule>();

		// Make it possible to find node based on hash-neighbourhood to remove duplicate nodes on hash
		mapHashNodeLookup = new HashMap<String, DefaultMutableTreeNode>();
		
		// Root nodes numbers created
		availableRootNodes = new HashMap<Integer, DefaultMutableTreeNode>();
		
		// Init tree
		treeRootNode = new DefaultMutableTreeNode("Rules");
		treeModel = new DefaultTreeModel(treeRootNode);
		treeObject = new JTree(treeModel);
		
		generateTree();
		initGUI();
	}
	
	public RuleEditor(){
		this(null);
	}

	private void generateTree() {
		
		ArrayList<Integer> possibleStates = rulebook.getPossibleStates();
		Collections.sort(possibleStates);
		
		// Generate visible root nodes
		for(Integer i : possibleStates){
			addRootNode(i);
		}
		
		// Populate visible root nodes
		for(HashRule r : rulebook.getHashRules().values()){
			addRuleToTree(r);
		}
		
	}


	private void addRootNode(Integer i) {
		DefaultMutableTreeNode root = (DefaultMutableTreeNode)treeModel.getRoot();
		DefaultMutableTreeNode newNode = new DefaultMutableTreeNode("Rule "+i);
		
		treeModel.insertNodeInto(newNode, root, root.getChildCount());
		
		// Add lookup for made rootNodes
		availableRootNodes.put(i, newNode);
		
	}

	private void addRuleToTree(HashRule r) {
		
		if(mapHashNodeLookup.containsKey(r.getNeighbourhood())){
			// Update nodes with same hash
			DefaultMutableTreeNode nodeToUpdate = mapHashNodeLookup.get(r.getNeighbourhood());
			nodeToUpdate.setUserObject(r.getDescription());
			treeModel.nodeChanged(nodeToUpdate);
			
		} else {
			// Add new node with new hash
			DefaultMutableTreeNode parentNode, newNode;

			int nodeNumber = (int)r.getNextState();
			
			// Create root node for rules in nodeNumer
			if(availableRootNodes.containsKey(nodeNumber) == false){
				addRootNode(nodeNumber);
			}
			
			parentNode = availableRootNodes.get(nodeNumber);
			newNode = new DefaultMutableTreeNode(r.getDescription());
			
			// Add new node to treeModel
			treeModel.insertNodeInto(newNode, parentNode, treeModel.getChildCount(parentNode));
			
			// Add node to reverse hash lookup, so HashRule can be updated when node changes
			mapHashRuleNodeLookup.put(newNode, r);
			
			// Add lookup to remove future collisions
			mapHashNodeLookup.put(r.getNeighbourhood(), newNode);
		}
	
		
	}
	
	public void addRule(){
		String hash = "";
		for(int y = 0; y<3; y++){
			for(int z = 0; z<3; z++){
				for(int x = 0; x<3; x++){
					hash += textFields[x][z][y].getText()+"-";
				}
			}
		}
		// Checks if become field is a number (should be)
		try{
			Integer nextState = Integer.parseInt(becomeField.getText());

			HashRule rule = new HashRule(hash, nextState, ruleName.getText());
			rulebook.addHashRule(rule, true);
			
			addRuleToTree(rule);
			
		} catch (NumberFormatException nfe) {
			System.err.println("not a number");
		}
	}
	
	public void updateTextFields(HashRule rule){
		String[] hash = rule.getNeighbourhood().split("-");
		
		// Update textfields
		for(int y = 0; y<3; y++){
			for(int z = 0; z<3; z++){
				for(int x = 0; x<3; x++){
					textFields[x][z][y].setText(hash[y*9+z*3+x]);
				}
			}
		}
		
		// Update next state field
		becomeField.setText(""+(int)rule.getNextState());
		
		// Update description field
		ruleName.setText(rule.getDescription());
	}
	
	public static void main(String[] args) {
		new RuleEditor();
	}


	@Override
	public void actionPerformed(ActionEvent a) {
		// Add a rule
		if(a.getSource().equals(saveRuleButton) || (a.getSource() instanceof JTextFieldClearOnFocus)){
			addRule();
		}
		
		// Delete a rule
		if(a.getSource().equals(deleteRuleButton)){
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) treeObject.getLastSelectedPathComponent();
			Object hashRule = mapHashRuleNodeLookup.get(node);
			
			if(hashRule instanceof HashRule){
				//Remove rule
				rulebook.removeHashRule((HashRule)hashRule);

				// Remove treeNode
				treeModel.removeNodeFromParent(node);
				
				mapHashNodeLookup.remove(((HashRule)hashRule).getNeighbourhood());
				mapHashRuleNodeLookup.remove(node);
			}
		}
		
		// Changed rule list
		if(a.getSource().equals(ruleFileList)){
			JComboBox cb = (JComboBox)a.getSource();
	        String ruleFile = (String)cb.getSelectedItem();
	        
	        useRuleFile(ruleFile);
	        
	        
		}
	}
	

	private void useRuleFile(String ruleFile) {
		// Not same rules, change
		if(rulebook.getRuleFile().equals(ruleFile) == false){
			if(game.getSubRuleBook() != null && game.getSubRuleBook().getRuleFile() == ruleFile){
				rulebook = game.getSubRuleBook();
			} else {
				rulebook = game.getRuleBook();
			}
			
			rulebook.loadRules();
			
			//clear everything
			clearTree();
			//Load in everything 
			generateTree();
		}
	}

	private void clearTree() {
		mapHashRuleNodeLookup.clear();
		mapHashNodeLookup.clear();
		availableRootNodes.clear();
		treeRootNode.removeAllChildren();
		treeModel.reload();
	}

	@Override
	public void valueChanged(TreeSelectionEvent tse) {
		DefaultMutableTreeNode node = (DefaultMutableTreeNode) treeObject.getLastSelectedPathComponent();
		
		Object rule = mapHashRuleNodeLookup.get(node);
		
		if(rule instanceof HashRule){
			updateTextFields((HashRule)rule);
		}
	}

	private void initGUI() {
		textFields = new JTextFieldClearOnFocus[3][3][3];
		
		int textFieldWidth = 20;
		
		if(System.getProperty("os.name").startsWith("Mac OS X")){
			textFieldWidth = 30;
		}
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setLocationRelativeTo(null);
		setPreferredSize(new Dimension(600, 400));
		pack();
		getContentPane().setLayout(null);
		
		JLabel lblRuleName = new JLabel("Rule name:");
		lblRuleName.setBounds(10, 11, 70, 14);
		getContentPane().add(lblRuleName);
		
		ruleName = new JTextField();
		ruleName.setBounds(90, 8, 334, 20);
		getContentPane().add(ruleName);
		ruleName.setColumns(10);
		ruleName.setText("");
		
		ruleFileList = new JComboBox(ruleFiles);
		ruleFileList.setSelectedIndex(0);
		ruleFileList.addActionListener(this);
		ruleFileList.setBounds(424, 8, 100, 20);
		getContentPane().add(ruleFileList);
		
		JLabel lblOver = new JLabel("Over:");
		lblOver.setBounds(20, 36, 46, 14);
		getContentPane().add(lblOver);
		
		JTextFieldClearOnFocus textField_0 = new JTextFieldClearOnFocus(this);
		textField_0.setBounds(10, 53, textFieldWidth, 20);
		getContentPane().add(textField_0);
		textField_0.setColumns(10);
		textFields[0][0][2] = textField_0;
		
		JTextFieldClearOnFocus textField_1 = new JTextFieldClearOnFocus(this);
		textField_1.setColumns(10);
		textField_1.setBounds(31, 53, textFieldWidth, 20);
		getContentPane().add(textField_1);
		textFields[1][0][2] = textField_1;
		
		JTextFieldClearOnFocus textField_2 = new JTextFieldClearOnFocus(this);
		textField_2.setColumns(10);
		textField_2.setBounds(52, 53, textFieldWidth, 20);
		getContentPane().add(textField_2);
		textFields[2][0][2] = textField_2;
		
		JTextFieldClearOnFocus textField_3 = new JTextFieldClearOnFocus(this);
		textField_3.setColumns(10);
		textField_3.setBounds(10, 74, textFieldWidth, 20);
		getContentPane().add(textField_3);
		textFields[0][1][2] = textField_3;
		
		JTextFieldClearOnFocus textField_4 = new JTextFieldClearOnFocus(this);
		textField_4.setColumns(10);
		textField_4.setBounds(31, 74, textFieldWidth, 20);
		getContentPane().add(textField_4);
		textFields[1][1][2] = textField_4;
		
		JTextFieldClearOnFocus textField_5 = new JTextFieldClearOnFocus(this);
		textField_5.setColumns(10);
		textField_5.setBounds(52, 74, textFieldWidth, 20);
		getContentPane().add(textField_5);
		textFields[2][1][2] = textField_5;
		
		JTextFieldClearOnFocus textField_6 = new JTextFieldClearOnFocus(this);
		textField_6.setColumns(10);
		textField_6.setBounds(10, 95, textFieldWidth, 20);
		getContentPane().add(textField_6);
		textFields[0][2][2] = textField_6;
		
		JTextFieldClearOnFocus textField_7 = new JTextFieldClearOnFocus(this);
		textField_7.setColumns(10);
		textField_7.setBounds(31, 95, textFieldWidth, 20);
		getContentPane().add(textField_7);
		textFields[1][2][2] = textField_7;
		
		JTextFieldClearOnFocus textField_8 = new JTextFieldClearOnFocus(this);
		textField_8.setColumns(10);
		textField_8.setBounds(52, 95, textFieldWidth, 20);
		getContentPane().add(textField_8);
		textFields[2][2][2] = textField_8;
		
		JLabel lblPlane = new JLabel("Plane:");
		lblPlane.setBounds(20, 126, 46, 14);
		getContentPane().add(lblPlane);
		
		JTextFieldClearOnFocus textField_9 = new JTextFieldClearOnFocus(this);
		textField_9.setColumns(10);
		textField_9.setBounds(10, 143, textFieldWidth, 20);
		getContentPane().add(textField_9);
		textFields[0][0][1] = textField_9;
		
		JTextFieldClearOnFocus textField_10 = new JTextFieldClearOnFocus(this);
		textField_10.setColumns(10);
		textField_10.setBounds(31, 143, textFieldWidth, 20);
		getContentPane().add(textField_10);
		textFields[1][0][1] = textField_10;
		
		JTextFieldClearOnFocus textField_11 = new JTextFieldClearOnFocus(this);
		textField_11.setColumns(10);
		textField_11.setBounds(52, 143, textFieldWidth, 20);
		getContentPane().add(textField_11);
		textFields[2][0][1] = textField_11;
		
		JTextFieldClearOnFocus textField_12 = new JTextFieldClearOnFocus(this);
		textField_12.setColumns(10);
		textField_12.setBounds(10, 164, textFieldWidth, 20);
		getContentPane().add(textField_12);
		textFields[0][1][1] = textField_12;
		
		JTextFieldClearOnFocus textField_13 = new JTextFieldClearOnFocus(this);
		textField_13.setColumns(10);
		textField_13.setBounds(31, 164, textFieldWidth, 20);
		getContentPane().add(textField_13);
		textFields[1][1][1] = textField_13;
		
		JTextFieldClearOnFocus textField_14 = new JTextFieldClearOnFocus(this);
		textField_14.setColumns(10);
		textField_14.setBounds(52, 164, textFieldWidth, 20);
		getContentPane().add(textField_14);
		textFields[2][1][1] = textField_14;
		
		JTextFieldClearOnFocus textField_15 = new JTextFieldClearOnFocus(this);
		textField_15.setColumns(10);
		textField_15.setBounds(10, 185, textFieldWidth, 20);
		getContentPane().add(textField_15);
		textFields[0][2][1] = textField_15;
		
		JTextFieldClearOnFocus textField_16 = new JTextFieldClearOnFocus(this);
		textField_16.setColumns(10);
		textField_16.setBounds(31, 185, textFieldWidth, 20);
		getContentPane().add(textField_16);
		textFields[1][2][1] = textField_16;
		
		JTextFieldClearOnFocus textField_17 = new JTextFieldClearOnFocus(this);
		textField_17.setColumns(10);
		textField_17.setBounds(52, 185, textFieldWidth, 20);
		getContentPane().add(textField_17);
		textFields[2][2][1] = textField_17;
		
		JLabel lblUnder = new JLabel("Under:");
		lblUnder.setBounds(20, 216, 46, 14);
		getContentPane().add(lblUnder);
		
		JTextFieldClearOnFocus textField_18 = new JTextFieldClearOnFocus(this);
		textField_18.setColumns(10);
		textField_18.setBounds(10, 233, textFieldWidth, 20);
		getContentPane().add(textField_18);
		textFields[0][0][0] = textField_18;
		
		JTextFieldClearOnFocus textField_19 = new JTextFieldClearOnFocus(this);
		textField_19.setColumns(10);
		textField_19.setBounds(31, 233, textFieldWidth, 20);
		getContentPane().add(textField_19);
		textFields[1][0][0] = textField_19;
		
		JTextFieldClearOnFocus textField_20 = new JTextFieldClearOnFocus(this);
		textField_20.setColumns(10);
		textField_20.setBounds(52, 233, textFieldWidth, 20);
		getContentPane().add(textField_20);
		textFields[2][0][0] = textField_20;
		
		JTextFieldClearOnFocus textField_21 = new JTextFieldClearOnFocus(this);
		textField_21.setColumns(10);
		textField_21.setBounds(10, 254, textFieldWidth, 20);
		getContentPane().add(textField_21);
		textFields[0][1][0] = textField_21;
		
		JTextFieldClearOnFocus textField_22 = new JTextFieldClearOnFocus(this);
		textField_22.setColumns(10);
		textField_22.setBounds(31, 254, textFieldWidth, 20);
		getContentPane().add(textField_22);
		textFields[1][1][0] = textField_22;
		
		JTextFieldClearOnFocus textField_23 = new JTextFieldClearOnFocus(this);
		textField_23.setColumns(10);
		textField_23.setBounds(52, 254, textFieldWidth, 20);
		getContentPane().add(textField_23);
		textFields[2][1][0] = textField_23;
		
		JTextFieldClearOnFocus textField_24 = new JTextFieldClearOnFocus(this);
		textField_24.setColumns(10);
		textField_24.setBounds(10, 275, textFieldWidth, 20);
		getContentPane().add(textField_24);
		textFields[0][2][0] = textField_24;
		
		JTextFieldClearOnFocus textField_25 = new JTextFieldClearOnFocus(this);
		textField_25.setColumns(10);
		textField_25.setBounds(31, 275, textFieldWidth, 20);
		getContentPane().add(textField_25);
		textFields[1][2][0] = textField_25;
		
		JTextFieldClearOnFocus textField_26 = new JTextFieldClearOnFocus(this);
		textField_26.setColumns(10);
		textField_26.setBounds(52, 275, textFieldWidth, 20);
		getContentPane().add(textField_26);
		textFields[2][2][0] = textField_26;
		
		JLabel lblBecomes = new JLabel("Becomes:");
		lblBecomes.setBounds(85, 146, 60, 14);
		getContentPane().add(lblBecomes);
		
		becomeField = new JTextFieldClearOnFocus(this);
		becomeField.setColumns(10);
		becomeField.setBounds(106, 164, textFieldWidth, 20);
		getContentPane().add(becomeField);
		
		treeObject.setEditable(false);
		treeObject.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		treeObject.addTreeSelectionListener(this);
		treeObject.setShowsRootHandles(true);
		treeObject.setRootVisible(true);
		JScrollPane jsp = new JScrollPane(treeObject);
		jsp.setBounds(156, 39, 418, 312);
		getContentPane().add(jsp);
		
		saveRuleButton = new JButton("Save rule");
		saveRuleButton.setBounds(35, 306, 89, 23);
		saveRuleButton.addActionListener(this);
		getContentPane().add(saveRuleButton);
		
		deleteRuleButton = new JButton("Delete rule");
		deleteRuleButton.setBounds(31, 330, 100, 23);
		deleteRuleButton.addActionListener(this);
		getContentPane().add(deleteRuleButton);
		
		setFocusTraversalPolicy(new FocusTraversalOnArray(new Component[]{textField_0, textField_1, textField_2, textField_3, textField_4, textField_5, textField_6, textField_7, textField_8, textField_9, textField_10, textField_11, textField_12, textField_13, textField_14, textField_15, textField_16, textField_17, textField_18, textField_19, textField_20, textField_21, textField_22, textField_23, textField_24, textField_25, textField_26, becomeField, ruleName, saveRuleButton, deleteRuleButton, treeObject, getContentPane(), lblRuleName, lblOver, lblPlane, lblUnder, lblBecomes}));
		setVisible(true);
	}
}
