package ca.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;

import mechanics.Game;

import ca.CallBack;
import ca.Cell;

public class SimpleGUI extends JFrame implements ActionListener {
	
	private static final long serialVersionUID = 1L;
	private Game game;
	private JTextField[][] jtf;
	private JPanel jp;
	private JButton nextButton;
	private boolean firstRun = true;
	
	public SimpleGUI() {
		
		setLayout(new BorderLayout());
		game = new Game(null);
		
		jp = new JPanel();
		jp.setLayout(new GridLayout(Game.gridWidth+1, Game.gridDepth+1));
		jtf = new JTextField[Game.gridWidth+1][Game.gridDepth+1];
		
		JTextField corner = new JTextField("z\\x");
		corner.setHorizontalAlignment(JTextField.CENTER);
		jp.add(corner);
		
		for(int i=1; i<Game.gridWidth+1; i++){
			JTextField j = new JTextField();
			j.setText(""+(i-1));
			jtf[i][0] = j;
			j.setHorizontalAlignment(JTextField.CENTER);
			jp.add(j);
		}
		
		updateTextfields();
		
		add(jp,BorderLayout.CENTER);
		
		JPanel buttons = new JPanel();
		add(buttons, BorderLayout.SOUTH);
		
		nextButton = new JButton("Next step");
		nextButton.addActionListener(this);
		buttons.add(nextButton);
		
		setSize( 400,450 );
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setVisible(true);
		
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource()==nextButton){
			game.step();
		}
	}
	
	public void updateTextfields(){
		String text = "";
		for(int x=1; x<=Game.gridWidth; x++){
			for(int z=0; z<=Game.gridDepth; z++){
				if(z == 0 && firstRun){
					JTextField j = new JTextField();
					j.setText(""+(x-1));
					j.setHorizontalAlignment(JTextField.CENTER);
					jtf[0][x] = j;
					jp.add(j);
				} else if (z > 0) {
					// x and z switched places somewhere
					Integer c = game.getCellGrid().getCellState(z-1, 0, x-1);
					if(c != null){
						text = ""+c;
					} else {
						text = "X";
					}
					if(jtf[x][z]==null){
						jtf[x][z] = new JTextField();
						jtf[x][z].setHorizontalAlignment(JTextField.CENTER);
						jp.add(jtf[x][z]);
					}
					if(text.equals("X")){
						jtf[x][z].setBackground(Color.DARK_GRAY);
					} else {
						if(c > 0){
							jtf[x][z].setBackground(Color.WHITE);
						} else {
							jtf[x][z].setBackground(Color.LIGHT_GRAY);
						}
					}
					jtf[x][z].setText(text);
				}
			}
		}
		firstRun = false;
	}
	
	public static void main(String[] args) {
		new SimpleGUI();
	}

}
