package ca.gui;

import glapp.GLApp;
import glapp.GLCam;
import glapp.GLCamera;
import glapp.GLImage;
import glapp.GLShadowOnPlane;
import glmodel.GL_Vector;

import java.nio.FloatBuffer;
import java.util.Calendar;

import mechanics.Game;

import org.lwjgl.input.Keyboard;
import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL12;
import org.lwjgl.util.glu.GLU;

import ca.CallBack;
import ca.Cell;
import ca.CellStructures;
import ca.RuleBook;

public class Visualizer extends GLApp implements CallBack {
    // Handle for texture
    StateTextureArrayList textureHandler = null;
    int groundTextureHandle = 0;
    private float lengthToSides = 150.0f;
    // Light position: if last value is 0, then this describes light direction.  If 1, then light position.
    float lightPosition[]= { 0, lengthToSides/2, lengthToSides, 1f };
    // Camera position
    float[] cameraPos = {5f,30f,5f};
    private int[] skybox = null;

    // two cameras and a cam to move them around scene
    GLCamera camera1 = new GLCamera();
    GLCam cam = new GLCam(camera1);

    // shadow handler will draw a shadow on floor plane
    GLShadowOnPlane airplaneShadow;
    
    // shadow handler will draw a shadow on floor plane
	FloatBuffer bbmatrix = GLApp.allocFloats(16);
	
	Game game;
	private long lastInformation;
	private String information;
	private boolean showSubcells;
	private int[] hiddenStates = {0, 7};//, 3 , 4};
	private int[] seeThroughStates = {3, 4};
	
	public Visualizer(int gridWidth, int gridHeight, int gridDepth) {
		game = new Game(this, CellStructures.BUILDNING_START, gridWidth, gridHeight, gridDepth);
		showSubcells = (game.getCellGrid().getSubGrid() != null);
		//game.getSubRuleBook().loadRules("extra_sub_rules.txt", false);
	}
	
	public Visualizer(){
		this(6, 5, 6);
	}
	
/*
	public static void main(String args[]) {
    	Visualizer demo = new Visualizer();
    	
        demo.VSyncEnabled = true;
        demo.fullScreen = false;
        demo.displayWidth = 800;
        demo.displayHeight = 600;
        demo.window_title = "CA Tester";
        demo.run();  // will call init(), render(), mouse functions
        
    }
*/
	
	/**
     * Initialize the scene.  Called by GLApp.run()
     */
    public void setup()
    {
    	game.addCallBack(this);

        // setup and enable perspective
        setPerspective();
        
        // Create a light (diffuse light, ambient light, position)
        setLight( GL11.GL_LIGHT1,
        		new float[] { 1f, 1f, 1f, 1f },
        		new float[] { 0.5f, 0.5f, .53f, 1f },
        		new float[] { 1f, 1f, 1f, 1f },
        		lightPosition );

        // Create a directional light (light green, to simulate reflection off grass)
        setLight( GL11.GL_LIGHT2,
        		new float[] { 0.15f, 0.4f, 0.1f, 1.0f },  // diffuse color
        		new float[] { 0.0f, 0.0f, 0.0f, 1.0f },   // ambient
        		new float[] { 0.0f, 0.0f, 0.0f, 1.0f },   // specular
        		new float[] { 0.0f, -10f, 0.0f, 0f } );   // direction (pointing up)

        // enable lighting and texture rendering
        GL11.glEnable(GL11.GL_LIGHTING);
        GL11.glEnable(GL11.GL_TEXTURE_2D);

        // Enable alpha transparency (so text will have transparent background)
        GL11.glEnable(GL11.GL_BLEND);
        GL11.glEnable(GL11.GL_DEPTH_TEST);
        GL11.glEnable(GL11.GL_ALPHA_TEST);
        GL11.glBlendFunc(GL11.GL_SRC_ALPHA, GL11.GL_ONE_MINUS_SRC_ALPHA);

        // Load state textures
        textureHandler = getStateTextures();
        
        // Create texture for ground plane
        groundTextureHandle = makeTexture("resources/images/grass_texture_repeating_patter.jpg",true,true);
        
        // Set up skybox
        skybox = new int[6];

        skybox[1] = makeTexture(new GLImage("resources/images/skybox/forward.png"));
        skybox[0] = makeTexture(new GLImage("resources/images/skybox/behind.png"));
        skybox[3] = makeTexture(new GLImage("resources/images/skybox/left.png"));
        skybox[2] = makeTexture(new GLImage("resources/images/skybox/right.png"));
        skybox[4] = makeTexture(new GLImage("resources/images/skybox/up.png"));
        skybox[5] = makeTexture(new GLImage("resources/images/skybox/bottom.png"));

        // set camera 1 position
        camera1.setCamera(10,30,10, 0f,-1f,0f, 0,0,-1);
        //camera1.setCamera(0,4,15, 0,-.3f,-1, 0,1,0);
        
        // make a shadow handler
        // params:
        //		the light position,
        //		the plane the shadow will fall on,
        //		the color of the shadow,
        // 		this application,
        // 		the function that draws all objects that cast shadows
        airplaneShadow = new GLShadowOnPlane(lightPosition, new float[] {0f,5f,0f,3f}, null, this, method(this,"drawObjects"));
    }

    /**
     * set the field of view and view depth.
     */
    public static void setPerspective()
    {
        // select projection matrix (controls perspective)
        GL11.glMatrixMode(GL11.GL_PROJECTION);
        GL11.glLoadIdentity();
        // fovy, aspect ratio, zNear, zFar
        GLU.gluPerspective(50f,         // zoom in or out of view
                           aspectRatio, // shape of viewport rectangle
                           .1f,         // Min Z: how far from eye position does view start
                           500f);       // max Z: how far from eye position does view extend
        // return to modelview matrix
        GL11.glMatrixMode(GL11.GL_MODELVIEW);
    }
    
    /**
     * Render one frame.  Called by GLApp.run().
     */
    public void draw() {
        // user keystrokes adjust camera position
        cam.handleNavKeys((float)GLApp.getSecondsPerFrame());
        
        // clear depth buffer and color
        GL11.glClear(GL11.GL_COLOR_BUFFER_BIT | GL11.GL_DEPTH_BUFFER_BIT);

        
        // select model view for subsequent transforms
        GL11.glMatrixMode(GL11.GL_MODELVIEW);
        GL11.glLoadIdentity();
        
    	cam.render();
    	
    	renderSkypbox(cam);

        // draw the ground plane
        GL11.glPushMatrix();
        {
        	
        	
            GL11.glTranslatef(-lengthToSides/2, -0.5f, -lengthToSides/2); // down a bit
            GL11.glScalef(lengthToSides*2, .01f, lengthToSides*2);
            GL11.glBindTexture(GL11.GL_TEXTURE_2D, groundTextureHandle);
            GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_S, GL11.GL_REPEAT);
            GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_T, GL11.GL_REPEAT);
            
            GL11.glBegin(GL11.GL_QUADS);
            // Top Face
            GL11.glNormal3f( 0.0f, 1.0f, 0.0f);
            GL11.glTexCoord2f(-lengthToSides, lengthToSides); GL11.glVertex3f(-1.0f,  1.0f, -1.0f);	// Top Left
            GL11.glTexCoord2f(-lengthToSides, -lengthToSides); GL11.glVertex3f(-1.0f,  1.0f,  1.0f);	// Bottom Left
            GL11.glTexCoord2f(lengthToSides, -lengthToSides); GL11.glVertex3f( 1.0f,  1.0f,  1.0f);	// Bottom Right
            GL11.glTexCoord2f(lengthToSides, lengthToSides); GL11.glVertex3f( 1.0f,  1.0f, -1.0f);	// Top Right
            GL11.glEnd();
            
        }
        GL11.glPopMatrix();
        
        // invokes the drawObjects() method to create shadows for objects in the scene
        airplaneShadow.drawShadow();
        

        // draw the scene (after we draw the shadows, so everything layers correctly)
        drawObjects();

        // Place the light.  Light will move with the rest of the scene
        setLightPosition(GL11.GL_LIGHT1, lightPosition);

        print( 30, viewportH- 45, "Generation: "+game.getGeneration() + " | cells("+game.getCellsInGrid().size()+","+game.getCellsInSubGrid().size()+")");
        getText();
        
        /*
		// render some text using texture-mapped font
		print( 30, viewportH- 45, "Use arrow keys to navigate:");
        print( 30, viewportH- 80, "Left-Right arrows rotate camera", 1);
        print( 30, viewportH-100, "Up-Down arrows move camera forward and back", 1);
        print( 30, viewportH-120, "PageUp-PageDown move vertically", 1);
        */
        
    }
    
    public void setText(String text){
    	this.information = text;
    	this.lastInformation = Calendar.getInstance().getTimeInMillis();
    }
    
    public void getText(){
    	if(Calendar.getInstance().getTimeInMillis() - lastInformation < 3000){
    		print( 30, viewportH- 80, this.information);
    	}
    }

    public void drawObjects() {
    	updateCells();
    }

    public void mouseMove(int x, int y) {
    }

    public void mouseDown(int x, int y) {
    }

    public void mouseUp(int x, int y) {
    }

    public void keyDown(int keycode) {
    }

    public void keyUp(int keycode) {
    	if (Keyboard.KEY_SPACE == keycode) {
    		long startTime = System.currentTimeMillis();
    		game.step();
    		System.out.println("Time: "+(System.currentTimeMillis()-startTime)+" ms");
    		
    		for(Cell c : game.getCellsInGrid()){
				if(c.getY()==8 && (c.getState() == 1 || c.getState() == 2)){
					System.out.println(c+" | "+c.getHash());
				}
			}
    		
    		final int inactiveLights = game.getStateCount(8);
    		final int activeLights = game.getStateCount(9);
    		final int inactiveWireLength = game.getStateCount(10);
    		final int activeWireLength = game.getStateCount(11);
    		
    		final double percent = activeLights/(float)(inactiveLights+activeLights)*100.0;
    		
    		
    		System.out.println("Inactive lights: "+inactiveLights);

    		System.out.println("Active lights: "+activeLights);
    		
    		System.out.println("Inactive Wire length: "+inactiveWireLength);
    		System.out.println("Active Wire length: "+activeWireLength);
    		
    		System.out.println("Percent completed: "+percent);
    	}
    	if (Keyboard.KEY_L == keycode) {
    		game.getRuleBook().loadRules();
    		if(game.hasSubgrid()) game.getSubRuleBook().loadRules();
    		setText("Loading rules");
    	}
    	if (Keyboard.KEY_F == keycode) {
    		setText("Locking restart at "+game.getGeneration()+" generation");
    		game.setGenerationLock(game.getGeneration());
    	}
    	if (Keyboard.KEY_Q == keycode) {
    		setText("Reset generation lock");
    		game.setGenerationLock(0);
    	}
    	if (Keyboard.KEY_R == keycode) {
    		setText("Resetting generations");
    		game.clearCells();
    		game.resetGeneration();
    		setText("Reset generations");
    	}
    	
    	if (Keyboard.KEY_B == keycode) {
    		setText("Resetting generations");
    		if(game.getGeneration() == 0){
    			setText("Nothing to revert");
    		} else {
    			game.revertToGeneration(game.getGeneration()-1);
    			updateCells();
    			setText("Reverting one generation");
    		}
    	}
    	if (Keyboard.KEY_ESCAPE == keycode) {
    		System.exit(0);
    	}
    	if (Keyboard.KEY_Y == keycode) {
    		game.getSubRuleBook().loadRules("extra_sub_rules.txt", false);
    	}
    	if (Keyboard.KEY_H == keycode) {
    		game.toggleStructureIsHidden();
    	}
    	if (Keyboard.KEY_I == keycode) {
    		showSubcells = !showSubcells;
    		updateCells();
    		
    		setText("Showing "+(showSubcells?"sub":"")+"grid");
    	}
    }
    
    /**
     * Updates drawn cell, adds if not exists
     * @param cell
     */
    public void updateCell(Cell cell){
    	// Do not show hidden states
    	for(int hiddenStateIndex = 0; hiddenStateIndex<hiddenStates.length; hiddenStateIndex++){
	    	if((int)cell.getState() == hiddenStates[hiddenStateIndex]){
	    		return;
	    	}
    	}
    	
    	for(int seeThroughStateIndex = 0; seeThroughStateIndex<seeThroughStates.length; seeThroughStateIndex++){
    		if((int)cell.getState() == seeThroughStates[seeThroughStateIndex]){
    			if(game.structureIsHidden()) return;
    			
    			GL_Vector pos = cam.camera.Position;
    			GL_Vector c = cell.getPosition();
    			// Check if camera is 20 from block, if less do not display it
    			if(Math.sqrt( Math.pow(pos.x-c.x, 2) +  Math.pow(pos.y-c.y, 2) + Math.pow(pos.z-c.z, 2)) < 20.0){
    				return;
    			}
    			break;
    		}
    	}
    	
		GL11.glPushMatrix();
        {
        	
			GL11.glTranslatef(cell.getX(), cell.getY(), cell.getZ());
			GL11.glScalef(0.5f, 0.5f, 0.5f);
			
        	GL11.glBindTexture(GL11.GL_TEXTURE_2D, textureHandler.get((int)cell.getState()));
        	renderCube();
        	
        }
        GL11.glPopMatrix();
	}
    
	private void updateCells() {
		if(showSubcells && game.getCellGrid().getSubGrid() != null){
			for(Cell c : game.getCellsInSubGrid()){
				updateCell(c);
			}
		} else {
			for(Cell c : game.getCellsInGrid()){
				updateCell(c);
			}
		}
	}

	@Override
	public void methodToCallBack() {
		updateCells();
	}
	
    private StateTextureArrayList getStateTextures() {
    	if(textureHandler == null){
    		textureHandler = new StateTextureArrayList();
    	}
		return textureHandler;
	}

	public RuleBook getRuleBook() {
		return game.getRuleBook();
	}

	public Game getGame() {
		return game;
	}

	private void renderSkypbox(GLCam cam){
		 // Store the current matrix
        GL11.glPushMatrix();
        
        // Reset and transform the matrix.
       // GL11.glLoadIdentity();
        
        GL11.glTranslatef(0, lengthToSides/8, 0);
        
        // Enable/Disable features
        GL11.glPushAttrib(GL11.GL_ENABLE_BIT);
        GL11.glEnable(GL11.GL_TEXTURE_2D);
        GL11.glDisable(GL11.GL_DEPTH_TEST);
        GL11.glDisable(GL11.GL_LIGHTING);
        GL11.glDisable(GL11.GL_BLEND);

        // Just in case we set all vertices to white.
        GL11.glColor4f(1,1,1,1); 
        
        // Render the front quad
        GL11.glBindTexture(GL11.GL_TEXTURE_2D, skybox[0]);
        GL11.glBegin(GL11.GL_QUADS);
        GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_S, GL12.GL_CLAMP_TO_EDGE);
        GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_T, GL12.GL_CLAMP_TO_EDGE); 
        GL11.glTexCoord2f(0, 0);GL11.glVertex3f(  lengthToSides, -lengthToSides, -lengthToSides );
        GL11.glTexCoord2f(0, 1);GL11.glVertex3f(  lengthToSides,  lengthToSides, -lengthToSides );
        GL11.glTexCoord2f(1, 1);GL11.glVertex3f( -lengthToSides,  lengthToSides, -lengthToSides );
        GL11.glTexCoord2f(1, 0);GL11.glVertex3f( -lengthToSides, -lengthToSides, -lengthToSides );
        GL11.glEnd();

        // Render the left quad
        GL11.glBindTexture(GL11.GL_TEXTURE_2D, skybox[2]);
        GL11.glBegin(GL11.GL_QUADS);
        GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_S, GL12.GL_CLAMP_TO_EDGE);
        GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_T, GL12.GL_CLAMP_TO_EDGE); 
        GL11.glTexCoord2f(0, 0);GL11.glVertex3f(  lengthToSides, -lengthToSides,  lengthToSides );
        GL11.glTexCoord2f(0, 1);GL11.glVertex3f(  lengthToSides,  lengthToSides,  lengthToSides );
        GL11.glTexCoord2f(1, 1);GL11.glVertex3f(  lengthToSides,  lengthToSides, -lengthToSides );
        GL11.glTexCoord2f(1, 0);GL11.glVertex3f(  lengthToSides, -lengthToSides, -lengthToSides );
        GL11.glEnd();

        // Render the back quad
        GL11.glBindTexture(GL11.GL_TEXTURE_2D, skybox[1]);
        GL11.glBegin(GL11.GL_QUADS);
        GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_S, GL12.GL_CLAMP_TO_EDGE);
        GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_T, GL12.GL_CLAMP_TO_EDGE); 
        GL11.glTexCoord2f(0, 0);GL11.glVertex3f( -lengthToSides, -lengthToSides,  lengthToSides );
        GL11.glTexCoord2f(0, 1);GL11.glVertex3f( -lengthToSides,  lengthToSides,  lengthToSides );
        GL11.glTexCoord2f(1, 1);GL11.glVertex3f(  lengthToSides,  lengthToSides,  lengthToSides );
        GL11.glTexCoord2f(1, 0);GL11.glVertex3f(  lengthToSides, -lengthToSides,  lengthToSides );

        GL11.glEnd();

        // Render the right quad
        GL11.glBindTexture(GL11.GL_TEXTURE_2D, skybox[3]);
        GL11.glBegin(GL11.GL_QUADS);
        GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_S, GL12.GL_CLAMP_TO_EDGE);
        GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_T, GL12.GL_CLAMP_TO_EDGE); 
        GL11.glTexCoord2f(0, 0);GL11.glVertex3f( -lengthToSides, -lengthToSides, -lengthToSides );
        GL11.glTexCoord2f(0, 1);GL11.glVertex3f( -lengthToSides,  lengthToSides, -lengthToSides );
        GL11.glTexCoord2f(1, 1);GL11.glVertex3f( -lengthToSides,  lengthToSides,  lengthToSides );
        GL11.glTexCoord2f(1, 0);GL11.glVertex3f( -lengthToSides, -lengthToSides,  lengthToSides );
        GL11.glEnd();

        // Render the top quad
        GL11.glBindTexture(GL11.GL_TEXTURE_2D, skybox[4]);
        GL11.glBegin(GL11.GL_QUADS);
        GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_S, GL12.GL_CLAMP_TO_EDGE);
        GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_T, GL12.GL_CLAMP_TO_EDGE); 
        GL11.glTexCoord2f(0, 1);GL11.glVertex3f( -lengthToSides,  lengthToSides, -lengthToSides );
        GL11.glTexCoord2f(1, 1);GL11.glVertex3f(  lengthToSides,  lengthToSides, -lengthToSides );
        GL11.glTexCoord2f(1, 0);GL11.glVertex3f(  lengthToSides,  lengthToSides,  lengthToSides );
        GL11.glTexCoord2f(0, 0);GL11.glVertex3f( -lengthToSides,  lengthToSides,  lengthToSides );
        GL11.glEnd();
        
        // Render the bottom quad
        GL11.glBindTexture(GL11.GL_TEXTURE_2D, skybox[5]);
        GL11.glBegin(GL11.GL_QUADS);
        GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_S, GL12.GL_CLAMP_TO_EDGE);
        GL11.glTexParameteri(GL11.GL_TEXTURE_2D, GL11.GL_TEXTURE_WRAP_T, GL12.GL_CLAMP_TO_EDGE); 
        GL11.glTexCoord2f(1, 0);GL11.glVertex3f(  lengthToSides,  -lengthToSides,  lengthToSides );
        GL11.glTexCoord2f(0, 0);GL11.glVertex3f( -lengthToSides,  -lengthToSides,  lengthToSides );
        GL11.glTexCoord2f(0, 1);GL11.glVertex3f( -lengthToSides,  -lengthToSides, -lengthToSides );
        GL11.glTexCoord2f(1, 1);GL11.glVertex3f(  lengthToSides,  -lengthToSides, -lengthToSides );
        GL11.glEnd();

        
        // Restore enable bits and matrix
        GL11.glPopAttrib();
        GL11.glPopMatrix();
	}
}