package genetic;

import glmodel.GL_Vector;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Random;

import mechanics.CellGrid;
import mechanics.Game;
import ca.Cell;
import ca.CellStructures;
import ca.RuleBook;
import ca.rules.HashRule;


public class GeneticAlgorithms {
	
	private RuleBook rulebook;
	private SimpleDateFormat sdf;
	private Random rand;
	private ArrayList<CellOrder> finalOrder;
	
	
	public GeneticAlgorithms() {
		sdf = new SimpleDateFormat("yyyy-MM-dd HH-mm-ss");
		rulebook = new RuleBook();
		rand = new Random();
		finalOrder = new ArrayList<CellOrder>();
		
		spreadTest();
		
		/*
		for(int i=0; i<100; i++){
			System.out.println("Test "+i);
			spreadTest();
			System.out.println();
			System.out.println();
		}
		*/
		//climbTest();
		System.out.println("All finished");
	}
	
	private void spreadTest() {
		
		int startGeneration = 28;
		// Only grow one floor
		Game game = new Game(null, CellStructures.BUILDNING_START, 6, 5, 6);
		// Jump to finished buildning
		game.goForwardToGeneration(startGeneration);
		
		int startY = (int) GAInstance.findStartCell(game, 12).y;
		
		CellGrid subGrid = game.getCellGrid().getSubGrid();
		if(subGrid == null){
			System.out.println("Subgrid Null!");
			return;
		}
		
		// Find max/min values for lights in X and Z direction
		int maxX = 0, maxZ = 0, minX = subGrid.getGridWidth(), minZ = subGrid.getGridDepth();
		for(Cell c : game.getCellsInSubGrid()){
			// If cell of type inactive light
			if(c.getState() == 8){
				if(c.getX() > maxX){
					maxX = c.getX();
				} else if(c.getX() < minX){
					minX = c.getX();
				}
				if(c.getZ() > maxZ){
					maxZ = c.getZ();
				} else if(c.getZ() < minZ){
					minZ = c.getZ();
				}
			}
		}
		
		// All lights starts with inactive wire above, therfor all cells in state 3
		// on y level given by startCell, and between minX/maxX and minZ/maxZ is potensial
		// candidates for a HashRule. TEST THEM ALL!
		// measure them by a fitness function, and send the best through to the next generation
		HashMap<String, GL_Vector> hashesToCheck = new HashMap<String, GL_Vector>();
		
		for(Cell c : game.getCellsInSubGrid()){
			if(c.getY() == startY){
				if(c.getX() <= maxX && c.getX() >= minX && c.getZ() <= maxZ && c.getZ() >= minZ){
					if(c.getState() != 10) hashesToCheck.put(c.getHash(), c.getPosition());
				}
			}
		}
		
		System.out.println(hashesToCheck.size());
		
		if(hashesToCheck.size() == 0 ){
			System.out.println("NO CELLS");
			return;
		}
		
		boolean finished = false;
		ArrayList<HashRule> chosenRules = new ArrayList<HashRule>();
		
		HashMap<String, Integer> points = new HashMap<String, Integer>();
		int generation = startGeneration;
		
		long startTime = System.currentTimeMillis();
		Integer highestPoints = 0;
		String bestHash = "";
		
		while(finished == false){
			generation++;
			System.out.println("NEW GENERATION ===== "+generation);
		
			// For all possible hashes
			for(String hash : hashesToCheck.keySet()){
				game = new Game(null, CellStructures.BUILDNING_START, 6, 5, 6);
				// Jump to finished buildning
				game.goForwardToGeneration(startGeneration);
				
				// Add already chosen rules
				for(HashRule cs : chosenRules){
					game.getSubRuleBook().addHashRule(cs, false);
				}
				
				System.out.println("active rules: "+game.getSubRuleBook().getHashRules().size());
				
				// Get hash and add it to rulebook
				HashRule hashRule = new HashRule(hash, 10, "GA generated in generation "+startGeneration);
				game.getSubRuleBook().addHashRule(hashRule, false);
				
				
				// Try the rule for 30 generations
				for(int i=0; i<30; i++){
					// Test how successfull the rule was, save result in HashMap with position
					int inactiveLights = game.getStateCount(8);
					int activeLights = game.getStateCount(9);
					double percent = activeLights / (double)(activeLights+inactiveLights);
					
					//System.out.println("test: "+i+" | Percent completed: "+percent);
					game.step();
				}
				
				// Test how successfull the rule was, save result in HashMap with position
				int inactiveLights = game.getStateCount(8);
				int activeLights = game.getStateCount(9);
				int wireLength = game.getStateCount(10) + game.getStateCount(11);
				double percent = activeLights / (double)(activeLights+inactiveLights);
				
				System.out.println("G:"+generation+" | "+hash);
				System.out.println("Lights: "+activeLights+"/"+inactiveLights);
				System.out.println("Wire length: "+wireLength);
				System.out.println("Percent completed: "+percent);
				
				// If finished, stop
				if((int)percent == 1) finished = true;
				if(finished) System.out.println("FINISHED");
				
				// Calculate score
				Integer score = activeLights*10 - wireLength;
				System.out.println("Points: "+score);
				// Save score
				points.put(hash, score);
				
				// remove the rule from testgame
				game.getSubRuleBook().removeHashRule(hashRule, false);
				
			    // Always start from the finished building
				// does not work :( make new game each time then...
				//game.revertToGeneration(startGeneration);
			}
			
			
			// Find cell with highest points
			
			for(String hash : hashesToCheck.keySet()){
				if(points.get(hash) > highestPoints){
					highestPoints = points.get(hash);
					bestHash = hash;
					
				// This just makes it some random, making it more likely to pic one of the two last alike scores
				} else if(points.get(hash) == highestPoints && rand.nextBoolean()){
					highestPoints = points.get(hash);
					bestHash = hash;
				}
				
			}
			
			// Clear points
			points.clear();
			
			// Make rule for best cell, and remove it from checking in the future
			HashRule bestRule = new HashRule(bestHash, 10, "GA made in generation "+generation+" with p"+highestPoints);
			chosenRules.add(bestRule);
			hashesToCheck.remove(bestHash);
			game.getSubRuleBook().addHashRule(bestRule, false);
			rulebook.addHashRule(bestRule, false);
			
		}
		
		rulebook.saveRules("geneticAlgorithms/F" +highestPoints + " - "+sdf.format(new Date())+" gen"+generation+".txt");
		System.out.println("Solution found after "+(System.currentTimeMillis() - startTime) + " ms");
		
	}

	private void climbTest() {
		
		GAInstance[] instances = new GAInstance[CellOrder.values().length];
		
		rulebook = new RuleBook();
		
		int startGeneration = 58;
		boolean isFinished = false;
		
		for(int i=0; i<instances.length; i++){
			instances[i] = new GAInstance(i, startGeneration, 12, rulebook, 6, 13, 6);
		}
		
		for(int gen=0; gen<150; gen++){
			System.out.println("New generation: "+gen);
			
			// Try all directions
			for(int i=0; i<instances.length; i++){
				instances[i].tryDirection();
			}
			
			// Find best direction
			StateInformation bestState = instances[0].state;
			int bestIndex = 0;
			for(int i=1; i<instances.length; i++){
				if(instances[i].state.points > bestState.points){
					bestState = instances[i].state;
					bestIndex = i;
					// Favoritise sequence
				}/* else if(instances[i].state.points == bestState.points && rand.nextBoolean()){
					bestState = instances[i].state;
					bestIndex = i;
				}*/
			}
			if(instances[bestIndex].getRule() != null){
				System.out.println();
				System.out.println("Chose: " + CellOrder.values()[bestIndex] + " ("+instances[bestIndex].state.position+")");
				System.out.println();
				
				finalOrder.add(CellOrder.values()[bestIndex]);
				
				
				// Make a rule of the best direction
				rulebook.addHashRule(instances[bestIndex].getRule(), false);
				rulebook.saveRules("geneticAlgorithms/p" + bestState.points + " - "+sdf.format(new Date())+" "+bestIndex+" gen"+gen+".txt");
				for(int j=0; j<instances.length; j++){
					instances[j].getGame().getSubRuleBook().addHashRule(instances[bestIndex].getRule(), false);
				}
			}
			
			// Copy best state to all and run one step
			for(int k=0; k<instances.length; k++){
				instances[k].setState(new StateInformation(bestState));
				instances[k].step();
				
				if(instances[k].isFinished()){
					rulebook.saveRules("geneticAlgorithms/F" + bestState.points + " - "+sdf.format(new Date())+" "+bestIndex+" gen"+gen+".txt");
					try{
						// Create file 
						FileWriter fstream = new FileWriter("geneticAlgorithms/p" + bestState.points + " - "+sdf.format(new Date())+" "+bestIndex+" gen"+gen+"-order.txt");
						BufferedWriter out = new BufferedWriter(fstream);
						for(CellOrder co : finalOrder){
							out.write(co+"\n");
						}
						//Close the output stream
						out.close();
					}catch (Exception e){//Catch exception if any
						System.err.println("Error: " + e.getMessage());
					}
					isFinished = true;
				}
				
				
				if(isFinished) break;
			}
		}
	}

	public static void main(String[] args) {
		new GeneticAlgorithms();

	}


}
